@@ -473,7 +473,7 @@ def parse_dwd_composite_header(header):
                 out["ncol"] = int(dimstrings[1])
             if k == "LV":
                 lv = header[v[0] : v[1]].split()
-                out["nlevel"] = np.int(lv[0])
+                out["nlevel"] = int(lv[0])
                 out["level"] = np.array(lv[1:]).astype("float")
             if k == "MS":
                 locationstring = header[v[0] :].strip().split("<")[1].split(">")[0]
@@ -1195,7 +1195,7 @@ def open_radolan_dataset(filename_or_obj, **kwargs):
 def open_radolan_mfdataset(paths, **kwargs):
     """Open multiple RADOLAN files as a single dataset.
 
-    Needs `dask` package to be installed.
+    Needs ``dask`` package to be installed [1]_.
 
     Parameters
     ----------
@@ -1215,6 +1215,10 @@ def open_radolan_mfdataset(paths, **kwargs):
     Returns
     -------
     dataset : :py:class:`xarray:xarray.Dataset`
+
+    References
+    ----------
+    .. [1] https://docs.dask.org/en/latest/
     """
     raise_on_missing_xarray_backend()
     backend_kwargs = dict(
@@ -1222,4 +1226,6 @@ def open_radolan_mfdataset(paths, **kwargs):
         copy=kwargs.pop("copy", False),
     )
     kwargs["backend_kwargs"] = backend_kwargs
+    if kwargs.get("concat_dim", False):
+        kwargs["combine"] = "nested"
     return xr.open_mfdataset(paths, engine="radolan", **kwargs)
