@@ -1,18 +1,29 @@
-# Copyright (c) 2011-2020, wradlib developers.
+# Copyright (c) 2011-2021, wradlib developers.
 # Distributed under the MIT License. See LICENSE.txt for more info.
 
 import contextlib
 import gc
-import io as sio
 
-import h5py
 import numpy as np
 import pytest
 import xarray as xr
 
 from wradlib import io, util
 
-from . import has_data, requires_data
+from . import (
+    get_wradlib_data_file,
+    h5py,
+    has_data,
+    requires_data,
+    requires_h5netcdf,
+    requires_h5py,
+    requires_netcdf,
+)
+
+
+@pytest.fixture(params=["file", "filelike"])
+def file_or_filelike(request):
+    return request.param
 
 
 def create_a1gate(i):
@@ -182,10 +193,10 @@ def create_dset_how(i, nrays=360):
 
 def create_dset_where(i, nrays=360):
     return {
-        "a1gate": np.array([create_a1gate(i)], dtype=np.int),
+        "a1gate": np.array([create_a1gate(i)], dtype=np.int_),
         "elangle": np.array([i + 0.5], dtype=np.float32),
-        "nrays": np.array([nrays], dtype=np.int),
-        "nbins": np.array([100], dtype=np.int),
+        "nrays": np.array([nrays], dtype=np.int_),
+        "nbins": np.array([100], dtype=np.int_),
         "rstart": np.array([0], dtype=np.float32),
         "rscale": np.array([1000], dtype=np.float32),
     }
@@ -223,11 +234,16 @@ def create_dataset(i, type=None, nrays=360):
     attrs = {}
     attrs["scale_factor"] = what["gain"]
     attrs["add_offset"] = what["offset"]
-    if type == "GAMIC":
-        attrs["add_offset"] -= 0.5
     attrs["_FillValue"] = what["nodata"]
-    attrs["coordinates"] = b"elevation azimuth range"
     attrs["_Undetect"] = what["undetect"]
+
+    if type == "GAMIC":
+        attrs["add_offset"] -= 0.5 + 127.5 / 254
+        attrs["scale_factor"] = 127.5 / 254
+        attrs["_FillValue"] = what["undetect"]
+        attrs["_Undetect"] = what["undetect"]
+
+    attrs["coordinates"] = b"elevation azimuth range"
     ds = xr.Dataset({"DBZH": (["azimuth", "range"], create_data(nrays=nrays), attrs)})
     return ds
 
@@ -245,28 +261,15 @@ def create_coords(i, nrays=360):
     return ds
 
 
-@pytest.fixture(params=["file", "filelike"])
-def file_or_filelike(request):
-    return request.param
-
-
-@contextlib.contextmanager
-def get_wradlib_data_file(file, file_or_filelike):
-    datafile = util.get_wradlib_data_file(file)
-    if file_or_filelike == "filelike":
-        _open = open
-        if datafile[-3:] == ".gz":
-            gzip = util.import_optional("gzip")
-            _open = gzip.open
-        with _open(datafile, mode="r+b") as f:
-            yield sio.BytesIO(f.read())
-    else:
-        yield datafile
-
-
 @contextlib.contextmanager
 def get_measured_volume(file, loader, format, fileobj):
-    # h5file = util.get_wradlib_data_file(file)
+    if loader == "noloader":
+        pass
+    elif loader == "netcdf4":
+        pytest.importorskip("netCDF4")
+    else:
+        pytest.importorskip(loader)
+        pytest.importorskip("h5netcdf")
     with get_wradlib_data_file(file, fileobj) as h5file:
         yield io.xarray.open_odim(h5file, loader=loader, flavour=format)
 
@@ -275,6 +278,15 @@ def get_measured_volume(file, loader, format, fileobj):
 def get_synthetic_volume(name, get_loader, file_or_filelike, **kwargs):
     import tempfile
 
+    if get_loader == "noloader":
+        pass
+    elif get_loader == "netcdf4":
+        pytest.importorskip("netCDF4")
+    else:
+        pytest.importorskip(get_loader)
+        pytest.importorskip("h5netcdf")
+    pytest.importorskip("h5py")
+
     tmp_local = tempfile.NamedTemporaryFile(suffix=".h5", prefix=name).name
     if "gamic" in name:
         format = "GAMIC"
@@ -923,6 +935,7 @@ class DataVolume(DataTimeSeries):
         del vol
         gc.collect()
 
+    @requires_h5py
     def test_odim_output(self, get_loader, file_or_filelike):
         if get_loader == "netcdf4" and self.format == "GAMIC":
             pytest.skip("gamic needs hdf-based loader")
@@ -936,6 +949,7 @@ class DataVolume(DataTimeSeries):
         del vol
         gc.collect()
 
+    @requires_netcdf
     def test_cfradial2_output(self, get_loader, file_or_filelike):
         if get_loader == "netcdf4" and self.format == "GAMIC":
             pytest.skip("gamic needs hdf-based loader")
@@ -951,6 +965,7 @@ class DataVolume(DataTimeSeries):
         del vol
         gc.collect()
 
+    @requires_netcdf
     def test_netcdf_output(self, get_loader, file_or_filelike):
         if get_loader == "netcdf4" and self.format == "GAMIC":
             pytest.skip("gamic needs hdf-based loader")
@@ -973,6 +988,10 @@ class MeasuredDataVolume(DataVolume):
         with get_measured_volume(self.name, loader, self.format, fileobj) as vol:
             yield vol
 
+    @property
+    def data(self):
+        return self._data
+
 
 class SyntheticDataVolume(DataVolume):
     @contextlib.contextmanager
@@ -982,6 +1001,8 @@ class SyntheticDataVolume(DataVolume):
 
 
 @requires_data
+@requires_h5netcdf
+@requires_netcdf
 class TestKNMIVolume(MeasuredDataVolume):
     if has_data:
         name = "hdf5/knmi_polar_volume.h5"
@@ -1008,13 +1029,17 @@ class TestKNMIVolume(MeasuredDataVolume):
         azimuths = [360] * sweeps
         ranges = [320, 240, 240, 240, 240, 340, 340, 300, 300, 240, 240, 240, 240, 240]
 
-        data = io.read_generic_hdf5(util.get_wradlib_data_file(name))
-
         dsdesc = "dataset{}"
         mdesc = "data{}"
 
+    @property
+    def data(self):
+        return io.read_generic_hdf5(util.get_wradlib_data_file(self.name))
+
 
 @requires_data
+@requires_h5netcdf
+@requires_netcdf
 class TestGamicVolume(MeasuredDataVolume):
     if has_data:
         name = "hdf5/DWD-Vol-2_99999_20180601054047_00.h5"
@@ -1039,11 +1064,13 @@ class TestGamicVolume(MeasuredDataVolume):
         azimuths = [361, 361, 361, 360, 361, 360, 360, 361, 360, 360]
         ranges = [360, 500, 620, 800, 1050, 1400, 1000, 1000, 1000, 1000]
 
-        data = io.read_generic_hdf5(util.get_wradlib_data_file(name))
-
         dsdesc = "scan{}"
         mdesc = "moment_{}"
 
+    @property
+    def data(self):
+        return io.read_generic_hdf5(util.get_wradlib_data_file(self.name))
+
 
 class TestSyntheticOdimVolume01(SyntheticDataVolume):
     name = "base_odim_data_00"
